import matplotlib.pyplot as plt
import numpy as np

def visualize_objects(env, title="Objects Distribution"):
    # Get all eatable foods
    foods = env.get_all_foods()
    food_positions = np.array([food.get_position() for food in foods if food.can_be_eaten()])
    # Get all alive organisms
    organisms = env.get_all_organisms()
    organisms_positions = np.array([org.get_position() for org in organisms if org.is_alive()])

    awareness_radii = np.array(
        [org.get_reaction_radius() for org in env.get_all_organisms()]
    )
    organisms_size = np.array([org.get_size() for org in env.get_all_organisms()])
    # life_span = np.array([org.get_life_span() for org in env.get_all_organisms()])

    plt.clf()
    # 檢查 organisms_positions 是否為空，不為空時繪圖
    if organisms_positions.size > 0:
        for i in range(len(organisms_positions)):
            circle = plt.Circle(
                (organisms_positions[i, 0], organisms_positions[i, 1]),
                organisms_size[i],
                color="blue",
                alpha=0.5,
            )
            plt.gca().add_artist(circle)

            circle = plt.Circle(
                (organisms_positions[i, 0], organisms_positions[i, 1]),
                awareness_radii[i],
                color="red",
                fill=False,
                alpha=0.5,
            )
            plt.gca().add_artist(circle)

    # 檢查 food_positions 是否為空，不為空時繪圖
    if food_positions.size > 0:
        plt.scatter(
            food_positions[:, 0], food_positions[:, 1], s=1, color="green", label="Food"
        )

    plt.xlim(0, env.get_width())
    plt.ylim(0, env.get_height())
    plt.xlabel("X")
    plt.ylabel("Y")
    plt.legend()
    plt.title(title)
    plt.draw()
    plt.pause(0.005)
    # input("Press Enter to continue...")




from simevopy import Environment, Genes, Organism
from utils.visualize import visualize_objects

def custom_lifespan_callback(organism):
    return 0

env = Environment(1000, 1000)
dna = chr(25) * 1 + chr(30) * 1 + chr(100) * 2
dna2 = chr(25) * 1 + chr(80) + chr(120) * 2

env.add_organism(Organism(Genes(dna), custom_lifespan_callback), 100, 100)
env.add_organism(Organism(Genes(dna2), custom_lifespan_callback), 80, 80)
visualize_objects(env)
env.simulate_iteration(100, on_each_iteration=visualize_objects)

input("Press Enter to continue...")



from simevopy import Environment
from utils.common import (
    distribute_food_randomly,
    remove_all_foods,
    reproduce_organisms,
    setup_base_organism,
)
from utils.visualize import visualize_objects


def distribute_food_like_oasis(env, food_count):
    distribute_food_randomly(env, int(food_count / 2), (800, 800), (1200, 1200))
    distribute_food_randomly(env, int(food_count / 2), (600, 600), (1500, 1500))


env = Environment(2000, 2000)

setup_base_organism(env, 50)

for i in range(200):
    print("=========================================")
    print(f"Gen {i} th")

    distribute_food_like_oasis(env, 100)

    visualize_objects(env, f"Organisms and Food Distribution (Generation {i})")
    env.simulate_iteration(
        100,
        on_each_iteration=lambda env: visualize_objects(
            env, f"Organisms and Food Distribution (Generation {i})"
        ),
    )
    print("After iteration, organism count: ", len(env.get_all_organisms()))


    reproduce_organisms(env)
    remove_all_foods(env)

input("Press Enter to continue...")



from skbuild import setup
from setuptools import find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="simevopy",
    version='0.1.17',
    description="SimEvo Python bindings",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    cmake_source_dir=".",
    url='https://github.com/YJack0000/SimEvo',
    author='YJack0000',
    author_email='yjack0000.cs12@nycu.edu.tw',
    license='MIT',
)
